﻿import mx.controls.TextInput;
import mx.controls.TextArea;
import mx.controls.RadioButtonGroup;
import mx.events.EventDispatcher;
import demo.undo.Memento;
import demo.undo.IOriginator;

class demo.views.ContactForm implements IOriginator {
// public properties:
	public var lastFieldModified:String;
	
// methods for EventDispatcher:
	public function addEventListener():Void {}
	public function removeEventListener():Void {};
	private var dispatchEvent:Function;
	
// ui elements:
	private var name:TextInput;
	private var number:TextInput;
	private var gender; // radio group can't be typed, MM bug
	private var notes:TextArea;
	
// initialization:
	private function ContactForm() {
		EventDispatcher.initialize(this);
	}
	
	private function onLoad():Void {
		configureUI();
		
		dispatchEvent({type:"change"});
	}
	private function configureUI():Void {
		// set up text inputs to broadcast modify events when they are changed:
		name.label.onSetFocus = number.label.onSetFocus = notes.label.onSetFocus = function() {
			this.oldText = this.text;
		}
		name.label.onKillFocus = number.label.onKillFocus = notes.label.onKillFocus = function() {
			if (this.oldText != this.text) {
				this._parent.dispatchEvent({type:"modify"});
			}
			delete(this.oldText);
		}
		
		// subscribe to events:
		name.addEventListener("modify",this);
		number.addEventListener("modify",this);
		notes.addEventListener("modify",this);
		
		gender.addEventListener("click",this);
	}
	
// public methods:
	// returns a generic object representing the current state:
	public function serialize():Object {
		return {name:name.text,number:number.text,gender:gender.selectedData,notes:notes.text}
	}
	
	// updates the state according to a state object:
	public function deserialize(p_data:Object):Void {
		name.text = p_data.name;
		number.text = p_data.number;
		gender.selectedData = p_data.gender;
		notes.text = p_data.notes;
	}

	// returns a memento object, required by IOriginator interface:
	public function createMemento():Memento {
		var state:Object = serialize();
		var label:String = (lastFieldModified) ? "Changed "+lastFieldModified : "Opened contact form";
		var m:Memento = new Memento(this,state,label);
		return m;
	}

	// updates state according to the passed memento object, required by IOriginator interface:
	public function setMemento(p_memento:Memento):Void {
		deserialize(p_memento.state);
	}
	
// private methods:
	// called whenever form data changes:
	private function modify(p_evtObj:Object):Void {
		// store the name of the item that was modified so we can create nice memento labels:
		lastFieldModified = (p_evtObj.target._name) ? p_evtObj.target._name : p_evtObj.target.groupName;
		dispatchEvent({type:"change"});
	}
	private function click(p_evtObj:Object):Void {
		modify(p_evtObj);
	}
}